@@ -127,4 +127,9 @@ class WeightedSPINE(AbstractEmbeddingModel):
 
     def is_using_edge_types(self) -> bool:
         """Returns whether the model is parametrized to use edge types."""
+        return False
+
+    @staticmethod
+    def is_stocastic() -> bool:
+        """Returns whether the model is stocastic and has therefore a random state."""
         return False
\ No newline at end of file
