@@ -18,7 +18,7 @@ class BoxEPyKeen(EntityRelationEmbeddingModelPyKeen):
         batch_size: int = 2**10,
         training_loop: Union[str, Type[TrainingLoop]
                              ] = "Stochastic Local Closed World Assumption",
-        random_seed: int = 42,
+        random_state: int = 42,
         enable_cache: bool = False
     ):
         """Create new PyKeen BoxE model.
@@ -54,7 +54,7 @@ class BoxEPyKeen(EntityRelationEmbeddingModelPyKeen):
             Can either be:
             - Stochastic Local Closed World Assumption
             - Local Closed World Assumption
-        random_seed: int = 42
+        random_state: int = 42
             Random seed to use while training the model
         enable_cache: bool = False
             Whether to enable the cache, that is to
@@ -68,7 +68,7 @@ class BoxEPyKeen(EntityRelationEmbeddingModelPyKeen):
             epochs=epochs,
             batch_size=batch_size,
             training_loop=training_loop,
-            random_seed=random_seed,
+            random_state=random_state,
             enable_cache=enable_cache
         )
 
