@@ -17,7 +17,7 @@ class TorusEPyKeen(EntityRelationEmbeddingModelPyKeen):
         batch_size: int = 2**10,
         training_loop: Union[str, Type[TrainingLoop]
                              ] = "Stochastic Local Closed World Assumption",
-        random_seed: int = 42,
+        random_state: int = 42,
         enable_cache: bool = False
     ):
         """Create new PyKeen TorusE model.
@@ -49,7 +49,7 @@ class TorusEPyKeen(EntityRelationEmbeddingModelPyKeen):
             Can either be:
             - Stochastic Local Closed World Assumption
             - Local Closed World Assumption
-        random_seed: int = 42
+        random_state: int = 42
             Random seed to use while training the model
         enable_cache: bool = False
             Whether to enable the cache, that is to
@@ -62,7 +62,7 @@ class TorusEPyKeen(EntityRelationEmbeddingModelPyKeen):
             epochs=epochs,
             batch_size=batch_size,
             training_loop=training_loop,
-            random_seed=random_seed,
+            random_state=random_state,
             enable_cache=enable_cache
         )
 
