@@ -38,7 +38,8 @@ class Siamese(TensorFlowEmbedder):
         learning_rate_plateau_patience: int = 2,
         use_mirrored_strategy: bool = False,
         optimizer: str = "nadam",
-        enable_cache: bool = False
+        enable_cache: bool = False,
+        random_state: int = 42
     ):
         """Create new sequence Siamese model.
 
@@ -73,6 +74,8 @@ class Siamese(TensorFlowEmbedder):
         enable_cache: bool = False
             Whether to enable the cache, that is to
             store the computed embedding.
+        random_state: Optional[int] = None
+            The random state to use if the model is stocastic.
         """
         self._relu_bias = relu_bias
 
@@ -86,7 +89,8 @@ class Siamese(TensorFlowEmbedder):
             batch_size=batch_size,
             optimizer=optimizer,
             use_mirrored_strategy=use_mirrored_strategy,
-            enable_cache=enable_cache
+            enable_cache=enable_cache,
+            random_state=random_state
         )
 
     @staticmethod
