@@ -11,10 +11,17 @@ from embiggen.utils.abstract_models import AbstractClassifierModel, AbstractEmbe
 class AbstractNodeLabelPredictionModel(AbstractClassifierModel):
     """Class defining an abstract node label prediction model."""
 
-    def __init__(self):
+    def __init__(self, random_state: Optional[int] = None):
+        """Create new abstract node-label prediction model.
+
+        Parameters
+        ---------------
+        random_state: Optional[int] = None
+            The random state to use if the model is stocastic.
+        """
         self._is_binary_prediction_task = None
         self._is_multilabel_prediction_task = None
-        super().__init__()
+        super().__init__(random_state=random_state)
 
     @staticmethod
     def requires_node_types() -> bool:
@@ -30,13 +37,14 @@ class AbstractNodeLabelPredictionModel(AbstractClassifierModel):
     def is_topological() -> bool:
         return False
 
-    def get_available_evaluation_schemas(self) -> List[str]:
+    @staticmethod
+    def get_available_evaluation_schemas() -> List[str]:
         """Returns available evaluation schemas for this task."""
         return [
             "Stratified Monte Carlo",
-            "Monte Carlo",
             "Stratified Kfold",
-            "Kfold"
+            "Monte Carlo",
+            "Kfold",
         ]
 
     def is_binary_prediction_task(self) -> bool:
@@ -54,6 +62,7 @@ class AbstractNodeLabelPredictionModel(AbstractClassifierModel):
         evaluation_schema: str,
         random_state: int,
         holdout_number: int,
+        number_of_holdouts: int,
         **holdouts_kwargs: Dict
     ) -> Tuple[Graph]:
         """Return train and test graphs tuple following the provided evaluation schema.
@@ -68,6 +77,8 @@ class AbstractNodeLabelPredictionModel(AbstractClassifierModel):
             The random state for the evaluation
         holdout_number: int
             The current holdout number.
+        number_of_holdouts: int
+            The total number of holdouts.
         holdouts_kwargs: Dict[str, Any]
             The kwargs to be forwarded to the holdout method.
         """
@@ -79,7 +90,7 @@ class AbstractNodeLabelPredictionModel(AbstractClassifierModel):
             )
         if evaluation_schema in ("Kfold", "Stratified Kfold"):
             return graph.get_node_label_kfold(
-                **holdouts_kwargs,
+                k=number_of_holdouts,
                 k_index=holdout_number,
                 use_stratification="Stratified" in evaluation_schema,
                 random_state=random_state,
