@@ -96,9 +96,9 @@ def get_model_or_stub(
         ):
             class StubClass(parent_class):
 
-                def __init__(self) :
+                def __init__(self, *args, **kwargs) :
                     """Raises a useful error message about this class."""
-                    super().__init__()
+                    super().__init__(**parent_class.smoke_test_parameters())
                     self.__class__.__name__ = model_class_name
                     other_candidates = self.find_available_models(
                         self.model_name(),
